/********************************
 *
 * NAME:
 * ----
 * randbReceiver/main.c
 *
 * AUTHORS:
 * -------
 * Alvaro Gutierrez -- aguti@etsit.upm.es
 * Alexandre Campo  -- acampo@ulb.ac.be
 *
 * REVISION HISTORY:
 *-----------------
 * 12/12/2008
 *
 * ADDITIONAL NOTES:
 * ----------------
 * A test for the e-randb board.
 * The e-puck reads data from the E-RANDB 
 */

#include <p30f6014a.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <btcom.h>
#include <e_randb.h>
#include <e_prox.h>
#include <e_uart_char.h>
#include <e_init_port.h>
#include <utility.h>

#define M_PI 3.141593
int main() {
	
	/* System Initialization */
	e_init_port();
	/* Init UART1 for bluetooth */
	e_init_uart1();
	/* Init E-RANDB board */
	e_init_randb(I2C);

	/* Wait for a command comming from bluetooth IRCOMTEST on pc directory*/
	/* Important issue when we are transmitting data and don't want
	 * the bluetooth stack stuck for programing mode */
	//btcomWaitForCommand('s');
	
	/* Range is tunable by software. 
	 * 0 -> Full Range (1m. approx depending on light conditions )
	 * 255 --> No Range (0cm. approx, depending on light conditions */
	e_randb_set_range(150);

	/* At some point we tought that the board could just take
	 * data en leave the calculations for the robot.
	 * At the moment, it is better to allow the board to do the calculations */
	/*e_randb_set_calculation(ON_ROBOT);*/
	e_randb_set_calculation(ON_BOARD);

	/* Store light conditions to use them as offset for the calculation 
	 * of the range and bearing */
	e_randb_store_light_conditions();

	/* Print on the bluetooth */
	char tmp2[50];	
	
	/* The counter for sending */
	unsigned int data = 0;
	double bearing = 0.0;
	unsigned int range = 0;
	unsigned int sensor = 0;
					
	while(1)
	{
		
		memset(tmp2, 0x0, 50);
				
		if(getselector()==0) {	// emitter to all
			
			data = 0x00FF;
			e_randb_send_all_data(data);
			sprintf(tmp2, "sent %u\r\n", data);		
			btcomSendString(tmp2);
			
		} else {				// receiver
			/* If something received */
			if(e_randb_get_if_received() != 0)
			{
				/* Get data */
				data = e_randb_get_data();
				/* Get bearing */
				bearing = e_randb_get_bearing();
				/*Get range */
				range = e_randb_get_range();
				/* Get the sensor that receive the message */
				sensor = e_randb_get_sensor();
				
				/* Print data received on the bluetooth */
				sprintf(tmp2,"%u %2f %u %u\r\n", data, (bearing*180/M_PI), range, sensor);
				btcomSendString(tmp2);
			}		
		}

	}
	return 0;
}
